package at.jku.ssw.graphviz;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;

// http://graphviz.org

class DotMakerHashtable {

	private final Element[] hashtable;
	private final StringBuilder b = new StringBuilder();
	private float width;
	private DecimalFormat decimalFormat;

	{
		DecimalFormatSymbols symbols = new DecimalFormatSymbols();
		symbols.setDecimalSeparator('.');
		decimalFormat = new DecimalFormat("#.#", symbols);
	}

	DotMakerHashtable(Element[] hashtable) {
		this(hashtable, 2.9f);
	}

	DotMakerHashtable(Element[] hashtable, float width) {
		this.hashtable = hashtable;
		this.width = width;
	}

	String getDot() {
		b.append("digraph MyGraph {\n");
		b.append("nodesep=.05;\nrankdir=LR;\n");
		table();
		chains();
		b.append("}");
		return b.toString();
	}

	private void table() {
		int nextId = 0;
		b.append("node [shape=record,width=.1,height=.1]\n\n");
		b.append(String.format("tab[width=%s,", decimalFormat.format(width)));
		b.append("label=\"");
		for (Element n : hashtable) {
			if (n != null) {
				if (n.next != null) {
					b.append(String.format("<f%d>", ++nextId));
				}
				if (!n.isDeleted) {
					b.append(escapeSpaces(n.key.toString()));
				} else {
					b.append("(deleted)");
				}
			} else {
				b.append('-');
			}
			b.append('|');
		}
		if (hashtable.length > 0) {
			b.delete(b.length() - 1, b.length());
		}
		b.append("\",");
		b.append("]");
		b.append(";\n\n");
	}

	private void chains() {
		int nextNodeId = 0;
		int nextId = 0;
		b.append("node [shape=box,fixedsize=true,labeloc=\"l\",");
		b.append(String.format("width=%s,height=.28]\n", decimalFormat.format(width)));
		for (Element n : hashtable) {
			if (n != null && n.next != null) {
				Element p = n.next;
				if (p != null) {
					++nextId;
				}
				while (p != null) {
					b.append(String.format("node%d[label=\"", ++nextNodeId));
					b.append(escapeSpaces(p.key.toString()));
					b.append("\"];\n");
					if (p == n.next) {
						b.append(String.format("tab:f%d:e->node%d:_:w;\n", nextId, nextNodeId));
					} else {
						b.append(String.format("node%d:_:e->node%d:_:w;\n", nextNodeId - 1, nextNodeId));
					}
					p = p.next;
				}
			}
		}
	}

	private String escapeSpaces(String s) {
		StringBuilder b = new StringBuilder(s);
		for (int i = b.length() - 1; i >= 0; i--) {
			if (b.charAt(i) == ' ') {
				b.insert(i, '\\');
			}
		}
		return b.toString();
	}
}